package com.me.asteroids.systems;

import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.me.asteroids.Graphics;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.model.Model;
import com.me.common.ecs.ComponentMapper;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.EntitySystem;

public class ModelRenderSystem extends EntitySystem {

    private ComponentMapper<ModelComponent> modelMapper;

    private ShapeRenderer renderer;

    public ModelRenderSystem(Engine engine, Graphics graphics) {
        super(engine, ModelComponent.class);
        this.modelMapper = engine.getComponentMapper(ModelComponent.class);
        this.renderer = graphics.getShapeRenderer();
    }

    @Override
    public void preProcess() {
        renderer.begin(ShapeRenderer.ShapeType.Line);
    }

    @Override
    public void processEntity(Entity entity, float dt) {
        Model model = modelMapper.get(entity).model;

        renderer.setColor(model.getColor());
        model.render(renderer);
    }

    @Override
    public void postProcess() {
        renderer.end();
    }

}
