package com.me.common.ecs;

import com.badlogic.gdx.utils.Array;

final class EntityManager {

    private Engine engine;

    protected Array<Entity> entities;
    protected Array<Entity> removedEntities;
    protected Array<Entity> toRefresh;

    EntityManager(Engine engine) {
        this.engine = engine;

        this.entities = new Array<>(false, 16);
        this.removedEntities = new Array<>(false, 16);
        this.toRefresh = new Array<>(false, 16);
    }

    public Entity create() {
        Entity entity;
        if (!removedEntities.isEmpty()) {
            entity = removedEntities.removeIndex(0);
        } else {
            entity = new Entity(engine);
        }
        entity.reset();
        entity.updateUniqueId();
        entities.add(entity);
        return entity;
    }

    public void update() {
        if (toRefresh.isEmpty()) {
            return;
        }

        for (Entity entity : toRefresh) {
            refreshEntity(entity);
            entity.pendingRefresh = false;
        }

        toRefresh.clear();
    }

    public void queueRefresh(Entity entity) {
        toRefresh.add(entity);
    }

    private void refreshEntity(Entity entity) {
        for (BaseSystem system : engine.systems) {
            if (!(system instanceof EntitySystem)) {
                continue;
            }

            ((EntitySystem) system).refresh(entity);
        }

        if (entity.removed) {
            engine.removeAllEntityComponents(entity.id);
            entities.removeValue(entity, true);
            removedEntities.add(entity);
        }
    }

}
